@@ -27,6 +27,7 @@ class _TMotorManState(Enum):
     IMPEDANCE = 1
     CURRENT = 2
     FULL_STATE = 3
+    SPEED = 4
 
 
 # the user-facing class that manages the motor.
@@ -68,9 +69,15 @@ class TMotorManager():
         self._motor_state_async = motor_state(0.0,0.0,0.0,0.0,0.0,0.0)
         self._command = MIT_command(0.0,0.0,0.0,0.0,0.0)
         self._control_state = _TMotorManState.IDLE
-        self._times_past_limit = 0
+        self._times_past_position_limit = 0
+        self._times_past_current_limit = 0
+        self._times_past_velocity_limit = 0
         self._angle_threshold = MIT_Params[self.type]['P_max'] - 2.0 # radians, only really matters if the motor's going super fast
+        self._current_threshold = MIT_Params[self.type]['T_max'] - 2.0 # A, only really matters if the motor's going super fast
+        self._velocity_threshold = MIT_Params[self.type]['V_max'] - 2.0 # radians, only really matters if the motor's going super fast
         self._old_pos = 0.0
+        self._old_curr = 0.0
+        self._old_vel = 0.0
 
         self._entered = False
         self._start_time = time.time()
@@ -138,11 +145,15 @@ class TMotorManager():
         Args:
             MIT_state: The MIT_Motor_State namedtuple with the most recent motor state.
         """
+        if MIT_state.error != 0:
+            raise RuntimeError('Driver board error for device: ' + self.device_info_string() + ": " + MIT_Params['ERROR_CODES'][MIT_state.error])
+
         now = time.time()
         dt = self._last_update_time - now
         self._last_update_time = now
         acceleration = (MIT_state.velocity - self._motor_state_async.velocity)/dt
 
+        # The "Current" supplied by the controller is actually current*Kt, which approximates torque.
         self._motor_state_async.set_state(MIT_state.position, MIT_state.velocity, MIT_state.current, MIT_state.temperature, MIT_state.error, acceleration)
         self._updated = True
 
@@ -166,20 +177,46 @@ class TMotorManager():
             warnings.warn("State update requested but no data from motor. Delay longer after zeroing, decrease frequency, or check connection. " + self.device_info_string(), RuntimeWarning)
         else:
             self._command_sent = False
-        # artificially extending the range of the position that we track
+
+        # artificially extending the range of the position, current, and velocity that we track
         P_max = MIT_Params[self.type]['P_max']
+        I_max =  MIT_Params[self.type]['T_max']/(MIT_Params[self.type]['NM_PER_AMP']*MIT_Params[self.type]['GEAR_RATIO'])
+        V_max =  MIT_Params[self.type]['V_max']
+
         old_pos = self._old_pos
+        old_curr = self._old_curr
+        old_vel = self._old_vel
+        
         new_pos = self._motor_state_async.position
-        thresh = self._angle_threshold
-        if (thresh <= new_pos and new_pos <= P_max) and (-P_max <= old_pos and old_pos <= -thresh):
-            self._times_past_limit -= 1
-        elif (thresh <= old_pos and old_pos <= P_max) and (-P_max <= new_pos and new_pos <= -thresh) :
-            self._times_past_limit += 1
+        new_curr = self._motor_state_async.current
+        new_vel = self._motor_state_async.velocity
+
+        thresh_pos = self._angle_threshold
+        thresh_curr = self._current_threshold
+        thresh_vel = self._velocity_threshold
+
+        if (thresh_pos <= new_pos and new_pos <= P_max) and (-P_max <= old_pos and old_pos <= -thresh_pos):
+            self._times_past_position_limit -= 1
+        elif (thresh_pos <= old_pos and old_pos <= P_max) and (-P_max <= new_pos and new_pos <= -thresh_pos) :
+            self._times_past_position_limit += 1
+        if (thresh_curr <= new_curr and new_curr <= I_max) and (-I_max <= old_curr and old_curr <= -thresh_curr):
+            self._times_past_current_limit -= 1
+        elif (thresh_curr <= old_curr and old_curr <= I_max) and (-I_max <= new_curr and new_curr <= -thresh_curr) :
+            self._times_past_current_limit += 1
+        if (thresh_vel <= new_vel and new_vel <= V_max) and (-V_max <= old_vel and old_vel <= -thresh_vel):
+            self._times_past_velocity_limit -= 1
+        elif (thresh_vel <= old_vel and old_vel <= V_max) and (-V_max <= new_vel and new_vel <= -thresh_vel) :
+            self._times_past_velocity_limit += 1
             
-        # update position
+        # update expanded state variables
         self._old_pos = new_pos
+        self._old_curr = new_curr
+        self._old_pos = new_pos
+
         self._motor_state.set_state_obj(self._motor_state_async)
-        self._motor_state.position += self._times_past_limit*2*P_max
+        self._motor_state.position += self._times_past_position_limit*2*P_max
+        self._motor_state.current += self._times_past_current_limit*2*I_max
+        self._motor_state.velocity += self._times_past_velocity_limit*2*V_max
         
         # send current motor command
         self._send_command()
@@ -189,21 +226,21 @@ class TMotorManager():
             self.csv_writer.writerow([self._last_update_time - self._start_time] + [self.LOG_FUNCTIONS[var]() for var in self.log_vars] + [data for data in self.extra_plots])
 
         self._updated = False
-        
-        
-
+    
     # sends a command to the motor depending on whats controlm mode the motor is in
     def _send_command(self):
         """Sends a command to the motor depending on whats controlm mode the motor is in. This method
         is called by update(), and should only be called on its own if you don't want to update the motor state info."""
         if self._control_state == _TMotorManState.FULL_STATE:
-            self._canman.MIT_controller(self.ID,self.type, self._command.position, self._command.velocity, self._command.kp, self._command.kd, self._command.current)
+            self._canman.MIT_controller(self.ID,self.type, self._command.position, self._command.velocity, self._command.kp, self._command.kd, self._command.current*MIT_Params[self.type]['NM_PER_AMP']*MIT_Params[self.type]['GEAR_RATIO'])
         elif self._control_state == _TMotorManState.IMPEDANCE:
             self._canman.MIT_controller(self.ID,self.type, self._command.position, self._command.velocity, self._command.kp, self._command.kd, 0.0)
         elif self._control_state == _TMotorManState.CURRENT:
-            self._canman.MIT_controller(self.ID, self.type, 0.0, 0.0, 0.0, 0.0, self._command.current)
+            self._canman.MIT_controller(self.ID, self.type, 0.0, 0.0, 0.0, 0.0, self._command.current*MIT_Params[self.type]['NM_PER_AMP']*MIT_Params[self.type]['GEAR_RATIO'])
         elif self._control_state == _TMotorManState.IDLE:
             self._canman.MIT_controller(self.ID,self.type, 0.0, 0.0, 0.0, 0.0, 0.0)
+        elif self._control_state == _TMotorManState.SPEED:
+            self._canman.MIT_controller(self.ID,self.type,0.0,self._command.velocity,0.0,self._command.kd,0.0)
         else:
             raise RuntimeError("UNDEFINED STATE for device " + self.device_info_string())
         self._last_command_time = time.time()
@@ -227,6 +264,20 @@ class TMotorManager():
         self._last_command_time = time.time()
 
     # getters for motor state
+    def get_temperature_celsius(self):
+        """
+        Returns:
+        The most recently updated qaxis current in amps
+        """
+        return self._motor_state.temperature
+    
+    def get_motor_error_code(self):
+        """
+        Returns:
+        The most recently updated qaxis current in amps
+        """
+        return self._motor_state.error
+
     def get_current_qaxis_amps(self):
         """
         Returns:
@@ -288,6 +339,7 @@ class TMotorManager():
         assert(isfinite(B) and MIT_Params[self.type]["Kd_min"] <= B and B <= MIT_Params[self.type]["Kd_max"])
         self._command.kp = K
         self._command.kd = B
+        self._command.velocity = 0.0
         self._control_state = _TMotorManState.IMPEDANCE
     
     # uses full MIT mode, will send whatever current command is set. 
@@ -320,6 +372,16 @@ class TMotorManager():
         """
         self._control_state = _TMotorManState.CURRENT
 
+    def set_speed_gains(self, kd=1.0):
+        """
+        Uses plain speed mode, will send 0.0 for position gain and for feed forward current.
+        
+        Args:
+            kd: The gain for the speed controller. Control law will be (v_des - v_actual)*kd = iq
+        """
+        self._command.kd = kd
+        self._control_state = _TMotorManState.SPEED
+
     # used for either impedance or MIT mode to set output angle
     def set_output_angle_radians(self, pos):
         """
@@ -340,6 +402,22 @@ class TMotorManager():
             raise RuntimeError("Attempted to send position command without gains for device " + self.device_info_string()) 
         self._command.position = pos
 
+    def set_output_velocity_radians_per_second(self, vel):
+        """
+        Used for either speed or full state feedback mode to set output velocity command.
+        Note, this does not send a command, it updates the TMotorManager's saved command,
+        which will be sent when update() is called.
+
+        Args:
+            vel: The desired output speed in rad/s
+        """
+        if np.abs(vel) >= MIT_Params[self.type]["V_max"]:
+            raise RuntimeError("Cannot control using speed mode for angles with magnitude greater than " + str(MIT_Params[self.type]["V_max"]) + "rad/s!")
+
+        if self._control_state not in [_TMotorManState.SPEED, _TMotorManState.FULL_STATE]:
+            raise RuntimeError("Attempted to send speed command without gains for device " + self.device_info_string()) 
+        self._command.velocity = vel
+
     # used for either current MIT mode to set current
     def set_motor_current_qaxis_amps(self, current):
         """
@@ -398,6 +476,15 @@ class TMotorManager():
         """
         self.set_output_angle_radians(pos/(MIT_Params[self.type]["GEAR_RATIO"]) )
 
+    def set_motor_velocity_radians_per_second(self, vel):
+        """
+        Wrapper for set_output_velocity that accounts for gear ratio to control motor-side velocity
+        
+        Args:
+            vel: The desired motor-side velocity in rad/s.
+        """
+        self.set_output_velocity_radians_per_second(vel/(MIT_Params[self.type]["GEAR_RATIO"]) )
+
     def get_motor_angle_radians(self):
         """
         Wrapper for get_output_angle that accounts for gear ratio to get motor-side angle
@@ -473,23 +560,42 @@ class TMotorManager():
     #  Characters" in VS Code, or the like to allow easy typing of ϕ, θ, and
     #  τ.
 
+    # controller variables
+    T = property(get_temperature_celsius, doc="temperature_degrees_C")
+    """Temperature in Degrees Celsius"""
+
+    e = property(get_motor_error_code, doc="temperature_degrees_C")
+    """Motor error code. 0 means no error."""
+
     # electrical variables
     i = property(get_current_qaxis_amps, set_motor_current_qaxis_amps, doc="current_qaxis_amps_current_only")
+    """Q-axis current in amps"""
 
     # output-side variables
     θ = property(get_output_angle_radians, set_output_angle_radians, doc="output_angle_radians_impedance_only")
-    θd = property (get_output_velocity_radians_per_second, doc="output_velocity_radians_per_second")
+    """Output angle in rad"""
+
+    θd = property (get_output_velocity_radians_per_second, set_output_velocity_radians_per_second, doc="output_velocity_radians_per_second")
+    """Output velocity in rad/s"""
+
     θdd = property(get_output_acceleration_radians_per_second_squared, doc="output_acceleration_radians_per_second_squared")
+    """Output acceleration in rad/s/s"""
+
     τ = property(get_output_torque_newton_meters, set_output_torque_newton_meters, doc="output_torque_newton_meters")
+    """Output torque in Nm"""
 
     # motor-side variables
     ϕ = property(get_motor_angle_radians, set_motor_angle_radians, doc="motor_angle_radians_impedance_only")
-    ϕd = property (get_motor_velocity_radians_per_second, doc="motor_velocity_radians_per_second")
+    """Motor-side angle in rad"""
+    
+    ϕd = property (get_motor_velocity_radians_per_second, set_motor_velocity_radians_per_second, doc="motor_velocity_radians_per_second")
+    """Motor-side velocity in rad/s"""
+
     ϕdd = property(get_motor_acceleration_radians_per_second_squared, doc="motor_acceleration_radians_per_second_squared")
+    """Motor-side acceleration in rad/s/s"""
+
     τm = property(get_motor_torque_newton_meters, set_motor_torque_newton_meters, doc="motor_torque_newton_meters")
-
-
-        
+    """Motor-side torque in Nm"""
 
 
 
